local name, addon = ...

local f = CreateFrame("Frame")
local login = true

addon['inits'] = addon['inits'] or {}
local pairs, ipairs = pairs, ipairs

addon['countTable'] = function(tab)
	local n = 0
	for _ in pairs(tab) do
		n = n + 1
	end
	return n
end

addon['debugTable'] = function(tab)
	if(type(tab) == 'table') then
		for k,v in pairs(tab) do
			print(k,v)
		end
	else
		print(tab)
	end
end

addon['colorize'] = function(text,color)
  return format("|cff%s%s|r",color,text)
end

addon['Print'] = function(message)
	SELECTED_CHAT_FRAME:AddMessage(tostring(message))
end

addon['IsTwink'] = function(tab, name)
	local realm = GetRealmName()
	if(addon['db']['sv']['profileKeys']) then
		for k,v in pairs(addon['db']['sv']['profileKeys']) do
			local n,m = strsplit(" - ",k,2)
			_,m = strsplit(" ",m,2)
			if(n) then
				if(n == name and m == realm) then
					return true
				end
			end
		end
	end
	return false
end

addon['buildMoneyString'] = function(money, color, tab)
	tab = tab or {}
	local iconSize = tab['iconSize'] or (addon['db']['global']['iconSize'] or 14)
	local goldicon = tab['goldicon'] or format("\124TInterface\\MoneyFrame\\UI-GoldIcon:%d:%d:1:0\124t", iconSize, iconSize)
	local silvericon = tab['silvericon'] or format("\124TInterface\\MoneyFrame\\UI-SilverIcon:%d:%d:1:0\124t", iconSize, iconSize)
	local coppericon = tab['coppericon'] or format("\124TInterface\\MoneyFrame\\UI-CopperIcon:%d:%d:1:0\124t", iconSize, iconSize)

	local moneystring
	local g,s,c
	local neg = false
	if(money <0) then 
		neg = true
		money = money * -1
	end
	g=floor(money/10000)
	s=floor((money-(g*10000))/100)
	c=money-s*100-g*10000
	moneystring = format("%s%s%s%s%s%s",g,goldicon,s,silvericon,c,coppericon)
	if(neg) then
		moneystring = format("-%s",moneystring)
	end
	if(color) then
		if(neg) then
			moneystring = addon['colorize'](moneystring,"ff0000")
		elseif(money ~= 0) then
			moneystring = addon['colorize'](moneystring,"44dd44")
		end
	end
	return moneystring
end

addon['itemInfo'] = function(id)
  local i = {}
  local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount,
itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(id)
  if(itemName) then
    i = {
      ['itemId'] = id,
      ['itemName'] = itemName,
      ['itemLink'] = itemLink,
      ['itemRarity'] = itemRarity,
      ['itemLevel'] = itemLevel,
      ['itemMinLevel'] = itemMinLevel,
      ['itemType'] = itemType,
      ['itemSubType'] = itemSubType,
      ['itemStackCount'] = itemStackCount,
      ['itemEquipLoc'] = itemEquipLoc,
      ['itemTexture'] = itemTexture,
      ['itemSellPrice'] = itemSellPrice
    }
  end
  return i
end

addon['getItemId'] = function(link)
  local id = nil
  if(link) then
    local _, _, itemString = string.find(link, "^|c%x+|H(.+)|h%[.*%]")
    _,id,_ = strsplit(":",itemString)
  end
  return id
end

addon['RegisterEvent'] = function(event, func)
	if(event and type(func) == 'function') then
		if(not f:IsEventRegistered(event)) then
			f:RegisterEvent(event)
		end
		addon[event] = addon[event] or {}
		addon[event][#addon[event] + 1] = func
	end
end
local throttles = {}
local schedules = {}
local scheduleFrame = CreateFrame("Frame")

local lastScheduleCheck = GetTime()
local ScheduleCheckDelay = .5
local function checkSchedule()
	if(lastScheduleCheck > GetTime()) then return end
	lastScheduleCheck = GetTime() + ScheduleCheckDelay
	for k = #schedules,1,-1	do
		v = schedules[k]
		if(v['lastcall'] + v['delay'] < GetTime()) then
			v['func'](v['event'], v['arg1'], v['arg2'])
			tremove(schedules, k)
		end
	end
	if(#schedules == 0) then
		scheduleFrame:Hide()
	end
end

scheduleFrame:SetScript("OnUpdate", checkSchedule)

local function scheduleThrottle(obj, arg1, arg2)
	local found,v = false
	for k = 1, #schedules do
		v = schedules[k]
		if(v['name'] == obj['name']) then
			v = obj
			v['arg1'] = arg1
			v['arg2'] = arg2
			found = true
			break
		end
	end
	if(not found) then
		schedules[#schedules + 1] = obj
		schedules[#schedules]['arg1'] = arg1
		schedules[#schedules]['arg2'] = arg2
	end
	scheduleFrame:Show()
end

local function eventThrottle(self, event, ...)
	for k = 1, #throttles do
		local v = throttles[k]
		if(v['event'] == event) then
			if(v['lastcall'] + v['delay'] < GetTime()) then
				v['lastcall'] = GetTime()
				v['func'](event, ...)
			else
				scheduleThrottle(v, ...)
			end
		end
	end
end

addon['RegisterEventThrottle'] = function(storename, event, delay, func)
	local found = 0
	for k = 1, #throttles do
		v = throttles[k]
		if(v['name'] == storename) then
			found = k
			break
		end
	end
	throttles[#throttles + 1] = {
		['name'] = storename,
		['event'] = event,
		['delay'] = delay,
		['func'] = func,
		['lastcall'] = GetTime() - delay,
	}
	if(found == 0) then
		addon['RegisterEvent'](event, eventThrottle)
	end
end

addon['round'] = function(num, idp)
  return math.floor(num * (10^(idp or 0)) + 0.5) / (10^(idp or 0))
end

addon['setupPrefs'] = function(childName, parentName, tab, defaults, profile, get, set)
	local typ = 'profile'
	tab[typ][childName] = tab[typ][childName] or CopyTable(defaults)
	local db = tab[typ][childName] 
	local options = {
		['type']='group',
		['childGroups']='tab',
		['args']={
			[childName] = {
				['name']=childName,
				['type']='group',
				['childGroups']='tab',
				['order']=1,
				['args']={
					['minimap'] = {
						['name']="Minimapicon",
						['type']='group',
						['inline']=true,
						['get'] = addon['minimapGet'],
						['set'] = addon['minimapSet'],
						['order'] = 100,
						['args']={
							['hide'] ={
								['type'] = 'toggle',
								['name'] = "Hide Minimapicon",
							},
							['minimapPos'] ={
								['type'] = 'range',
								['name'] = "Minimapicon Position",
								['min'] = 1,
								['max'] = 250,
								['step']=1,
							}
						}
					}
				},
				['plugins']={
				},
			}
		}
	}
	if(type(set) == 'function') then
		options['args'][childName]['set'] = set
	end
	if(type(get) == 'function') then
		options['args'][childName]['get'] = get
	end
	if(profile) then
		options['args']['profile'] = LibStub("AceDBOptions-3.0"):GetOptionsTable(tab)
	end
	if(not addon['IsAdded'](childName)) then
		if(childName == name) then
			options['args']['profile'] = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon['db'])
			options['args']['profile2'] = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon['cdb'])
			options['args']['profile2']['name'] = "Char "..options['args']['profile2']['name']
		end
		LibStub("AceConfig-3.0"):RegisterOptionsTable(childName, options)
		LibStub("AceConfigDialog-3.0"):AddToBlizOptions(childName, childName, parentName)
	end
	return options
end

addon['addCharDefaults'] = function(cname, scope, tab)
	addon['chardefaults'] = addon['chardefaults'] or {
		['global'] = {},
	}
	addon['chardefaults'][scope][cname] = CopyTable(tab)
end

local common = {
	['config'] = true,
}

local function handleInits(tab)
	for k, v in pairs(tab) do -- check for common inits
		if(common[k] and type(v) == 'function') then
			v()
		end
	end
	for k, v in pairs(tab) do -- check for inits

		if(type(v) == 'table') then
--			handleInits(v)
		elseif(type(v) == 'function' and not common[k]) then
--		print("=",k,v)
			if(type(k) == 'string' and addon['db']['global'][k.."toggle"] == nil) then addon['db']['global'][k.."toggle"] = true end
			if(addon['db']['global'][k.."toggle"] or type(k) ~= 'string') then
--				print("+",k, common[k], type(k))
				v()
			end
		elseif(not common[k]) then
			print("error", k, v)
		end
	end
	for k, v in pairs(tab) do -- check for nested inits
		if(type(v) == 'table') then
-- print("-",k, v)
			handleInits(v)
		elseif(type(v) == 'function') then
--			v()
		else
			print("error", k, v)
		end
	end
end

local function setupPrefs()
		for _, childName in ipairs(addon['childs']) do
			addon['AddConfigEntry']("toggle", childName.."toggle",childName,"De-/Aktiviert dieses AddOn nach dem nchsten UI-Reload.")
		end
end

local function init()
	addon['db'] = LibStub("AceDB-3.0"):New(name.."DB",addon['defaults'],true)
	addon['cdb'] = LibStub("AceDB-3.0"):New(name.."CharDB",addon['chardefaults'],true)
	handleInits(addon['inits'])
	setupPrefs()
end

local function onevent(self, event, arg1, ...)
	if(login and ((event == "ADDON_LOADED" and name == arg1) or (event == "PLAYER_LOGIN"))) then
		login = nil
		f:UnregisterEvent("ADDON_LOADED")
		f:UnregisterEvent("PLAYER_LOGIN")
		init()
	else
		if(type(addon[event]) == 'table') then
			for _,func in ipairs(addon[event]) do
				if(type(func) == 'function') then
					func(self, event, arg1, ...)
				end
			end
		end
	end
end
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("PLAYER_LOGIN")
f:SetScript("OnEvent", onevent)