local name, addon = ...
local self = CreateFrame("Frame")
local parentName = addon['parentName'] or "BrokerPack"
addon['inits'] = addon['inits'] or {}

addon['minimapGet'] = function(pref)
	return addon['db']['global']['ldbicons'][pref[#pref - 2]][pref[#pref]]
end

addon['minimapSet'] = function(pref,value)
	addon['db']['global']['ldbicons'][pref[#pref - 2]][pref[#pref]] = value
	addon['ldbiconUpdate'](pref[#pref - 2])
end

local function getPref(pref)
	return addon['db']['global'][pref[#pref]]
end

local function setPref(pref,value)
	addon['db']['global'][pref[#pref]] = value
end

addon['options'] = {
	['type']='group',
	['handler']=self,
	['get']=getPref,
	['set']=setPref,
	['childGroups']='tab',
	['args']={
		[parentName] = {
			['name'] = parentName,
			['type']='group',
			['handler']=self,
			['get']=getPref,
			['set']=setPref,
			['order']=1,
			['childGroups']='tab',
			['args']={
			}
		}
	}
}

local function AddConfigMenu(menuName, parentMenu)
	if(not parentMenu) then
		parentMenu = addon['options']
	end
	local menuHandler,menuChildGroups,menuGet,menuSet,menuOrder,menuDesc
	if(type(menuName) == 'table') then
		menuHandler = menuName['menuHandler'] or self
		menuChildGroups = menuName['childGroups'] or 'tree'
		menuGet = menuName['menuGet']  or getPref
		menuSet = menuName['menuSet'] or setPref
		menuOrder = menuName['order'] or 1
		menuDesc = menuName['desc']  or ''
		menuName = menuName['name']
	else
		menuHandler = self
		menuGet = 'getPref'
		menuSet = 'setPref'
		menuOrder = 1
		menuDesc = ''
	end
	parentMenu['args'][menuName] = {
		['type'] = 'group',
		['name'] = menuName,
		['desc'] = menuDesc,
		['childGroups'] = menuChildGroups,
		['handler'] = menuHandler,
		['get'] = menuGet,
		['set'] = menuSet,
		['order'] = menuOrder,
		['args']  = {},
	}
	return parentMenu['args'][menuName]
end

local function AddConfigEntry(...)
	local menu = select(10,...) or addon['options']['args'][parentName]
	if(menu) then
		menu['args'][select(2,...)] = {
			['type'] = select(1,...),
			['name'] = select(3,...),
			['desc'] = select(4,...),
			['order'] = select(5,...)
		}
		if(select(1,...) == 'range') then
			menu['args'][select(2,...)]['min'] =  select(6,...)
			menu['args'][select(2,...)]['max'] =  select(7,...)
			menu['args'][select(2,...)]['step'] =  select(8,...)
			menu['args'][select(2,...)]['isPercent'] =  select(9,...)
		elseif(select(1,...) == 'execute') then
			menu['args'][select(2,...)]['func'] =  select(6,...)
		elseif(select(1,...) == 'color') then
			menu['args'][select(2,...)]['get'] =  select(6,...)
			menu['args'][select(2,...)]['set'] =  select(7,...)
			menu['args'][select(2,...)]['hasAlpha'] = true
		elseif(select(1,...) == 'select') then
			menu['args'][select(2,...)]['values'] = select(6,...)
		end
	end
end

local function IsAdded(frame)
	local acd = LibStub("AceConfigDialog-3.0")['BlizOptions']
	for k, v in pairs(acd) do
		if(k == frame) then return true end
	end
	return false
end
addon['IsAdded'] = IsAdded

local function init()
	if(not addon['cfgDone'] and not IsAdded(parentName)) then
		if(name == addon['parentName']) then
			addon['options']['args']['profile'] = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon['db'])
			addon['options']['args']['profile2'] = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon['cdb'])
			addon['options']['args']['profile2']['name'] = "Char "..addon['options']['args']['profile2']['name']
		end
		LibStub("AceConfig-3.0"):RegisterOptionsTable(parentName, addon['options'])
		LibStub("AceConfigDialog-3.0"):AddToBlizOptions(parentName, parentName)
		addon['cfgDone'] = true
	end
end
addon['AddConfigEntry'] = AddConfigEntry
addon['AddConfigMenu'] = AddConfigMenu
addon['inits']['config'] = init